package asd.td4;

/**
 * Une classe pour les exercices 1,2 et 3 de la feuille 4.
 *  2002 Marc Gatano
 */
public class Liste {

    private Object premier; // la valeur de la cellule
    private Liste  reste;   // la suite du chanage

    // Constructeurs

    /**
     * Construit une liste vide.
     */
    public Liste() {
        this.premier = null;
        this.reste = null;
    }

    /**
     * Construit une liste  un lment, l'objet x.
     */
    public Liste(Object x) {
        this(x,new Liste());
    }

    /**
     * Construit une liste dont le premier lment est l'objet x
     * et dont la suite est la liste l.
     */
    public Liste(Object x, Liste l) {
        this.premier = x;
        this.reste = l;
    }

    // Primitives

    /**
     * Retourne le premier lment de la liste.
     */
    public Object premier() {
        return premier;
    }

    /**
     * Retourne le reste de la liste, c'est  dire la liste initiale
     * sans son premier lment.
     */
    public Liste reste() {
        return reste;
    }

    /**
     * Ajoute l'objet x en tte de la liste.
     */
    public void ajouter(Object x) {
        reste   = new Liste(premier,reste);
        premier = x;
    }

    /**
     * Supprime l'lment en tte de la liste.
     * Prcondition : la liste n'est pas vide.
     */
    public void supprimer() {
        premier = reste.premier;
        reste   = reste.reste;
    }

    /**
     * Teste si la liste est vide.
     */
    public boolean vide() {
        return reste == null;
    }

    // Non primitives

    /**
     * Retourne une chane de caractres reprsentant la liste.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public String toString() {
        if ( vide() ) {
            return "->";
        }
        String s = "-> " + premier();
        Liste l = reste();
        while ( ! l.vide() ) {
            s = s + " -> " + l.premier();
            l = l.reste();
        } 
        return s;
    }
        
    // NON-PRIMITIVES  CRIRE  l'AIDE DES PRIMITIVES

    /**
     * Retourne la longueur de la liste.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public int longueurIteratif() {
        Liste l = this;
        int i = 0;
        while ( ! l.vide() ) {
            l = l.reste();
            i++;
        }
        return i;
    }

    /**
     * Retourne la longueur de la liste.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public int longueurRecursif() {
        if ( vide() ) {
            return 0;
        }
        return 1 + reste().longueurRecursif();
    }

    /**
     * Retourne le i-ime lment de la liste. Si i > n o n est le
     * nombre d'lments de la liste, retourne le dernier lment de
     * la liste.
     * Prcondition : i > 0.et la liste n'est pas vide.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public Object elementIteratif(int i) {
        Liste l = this;
        while ( i > 1 && ! l.reste().vide() ) {
            l = l.reste();
            i--;
        }
        return l.premier();
    }

    /**
     * Retourne le i-ime lment de la liste. Si i > n o n est le
     * nombre d'lments de la liste, retourne le dernier lment de
     * la liste.
     * Prcondition : i > 0.et la liste n'est pas vide.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public Object elementRecursif(int i) {
        if ( i == 1 || reste().vide() ) {
            return premier();
        }
        return reste().elementRecursif(i-1);
    }

    /**
     * Retourne la position de la premire occurence de l'objet x
     * dans la liste (le premier lment de la liste tant  la
     * position 1). Si l'objet x n'est pas prsent dans la liste,
     * retourne 0.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public int positionIteratif(Object x) {
        Liste l = this;
        int i = 1;
        while ( ! l.vide() && ! x.equals(l.premier()) ) {
            l = l.reste();
            i++;
        }
        if ( ! l.vide() ) {
            return i;
        }
        return 0;
    }

    /**
     * Retourne la position de la premire occurence de l'objet x
     * dans la liste (le premier lment de la liste tant  la
     * position 1). Si l'objet x n'est pas prsent dans la liste,
     * retourne 0.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public int positionRecursif(Object x) {
        if ( vide() ) {
            return 0;
        }
        if ( x.equals(premier()) ) {
            return 1;
        }
        int i = reste().positionRecursif(x);
        if ( i != 0 ) {
            return i + 1;
        }
        return 0;
    }

    /**
     * Ajoute l'objet x aprs le i-me lment de la liste.
     * Si i = 0, ajoute l'objet x en tte de la liste.
     * Si i > n o n est le nombre d'lments de la liste, ajoute
     * l'objet x  la fin de la liste.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public void ajouterIteratif(Object x, int i) {
        Liste l = this;
        while ( ! l.vide() && i > 0 ) {
            l = l.reste();
            i--;
        }
        l.ajouter(x);
    }

    /**
     * Ajoute l'objet x aprs le i-me lment de la liste.
     * Si i = 0, ajoute l'objet x en tte de la liste.
     * Si i > n o n est le nombre d'lments de la liste, ajoute
     * l'objet x  la fin de la liste.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public void ajouterRecursif(Object x, int i) {
        if ( vide() || i == 0 ) {
            ajouter(x);
        }
        else {
            reste().ajouterRecursif(x,i-1);
        }
    }

    /**
     * Supprime le i-ime lment de la liste.
     * Si i > n o n est le nombre d'lments de la liste, ne fait rien.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public void supprimerIteratif(int i) {
        Liste l = this;
        while ( ! l.vide() && i > 1 ) {
            l = l.reste();
            i--;
        }
        if ( ! l.vide() ) {
            l.supprimer();
        }
    }

    /**
     * Supprime le i-ime lment de la liste.
     * Si i > n o n est le nombre d'lments de la liste, ne fait rien.
     * Complexit : O(n) o n est le nombre d'lments de la liste.
     */
    public void supprimerRecursif(int i) {
        if ( ! vide() ) {
            if ( i == 1 ) {
                supprimer();
            }
            else {
                reste().supprimerRecursif(i-1);
            }
        }
    }
}
